#!/usr/bin/env python
'''
Copyright (C) 2008 Nicolas Dufour - Association Inkscape-fr

##This extension allows you to draw crop marks in Inkscape.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''

import inkex
import simplestyle

def draw_SVG_line(x1, y1, x2, y2, width, name, parent):
    style = { 'stroke': '#000000', 'stroke-width':str(width), 'fill': 'none'}
    line_attribs = {'style':simplestyle.formatStyle(style),
                    inkex.addNS('label','inkscape'):name,
                    'd':'M '+str(x1)+','+str(y1)+' L '+str(x2)+','+str(y2)}
    inkex.etree.SubElement(parent, inkex.addNS('path','svg'), line_attribs)

def draw_SVG_dashedline(x1, y1, x2, y2, width, name, parent):
    style = { 'stroke': '#000000', 'stroke-width':str(width), 'fill': 'none',
              'stroke-miterlimit': '4', 'stroke-dasharray': '4, 2, 1, 2',
              'stroke-dashoffset': '0' }
    line_attribs = {'style':simplestyle.formatStyle(style),
                    inkex.addNS('label','inkscape'):name,
                    'd':'M '+str(x1)+','+str(y1)+' L '+str(x2)+','+str(y2)}
    inkex.etree.SubElement(parent, inkex.addNS('path','svg'), line_attribs)

def draw_SVG_RegCircles(cx, cy, r, name, colours, parent):
    for i in range(len(colours)):
        style = {'stroke':colours[i], 'stroke-width':str(r / len(colours)),
                 'fill':'none'}
        circle_attribs = {'style':simplestyle.formatStyle(style),
                          inkex.addNS('label','inkscape'):name,
                          'cx':str(cx), 'cy':str(cy),
                          'r':str((r / len(colours)) * (i + 0.5))}
        inkex.etree.SubElement(parent, inkex.addNS('circle','svg'),
                               circle_attribs)

def draw_SVG_RegMarks(cx, cy, r, rotate, name, colours, parent):
    for i in range(len(colours)):
        style = {'fill':colours[i], 'fill-opacity':'1', 'stroke':'none'}
        step = r
        stroke = r / len(colours)
        regoffset = stroke * i
        regmark_attribs = {'style':simplestyle.formatStyle(style),
                          inkex.addNS('label','inkscape'):name,
                          'd':'m '+str(- regoffset)+','+str(r)\
                          +' '+str(-stroke)+',0'\
                          +' '+str(step)+','+str(-r)\
                          +' '+str(-step)+','+str(-r)\
                          +' '+str(stroke)+',0'\
                          +' '+str(step)+','+str(r)\
                          +' '+str(-step)+','+str(r)\
                          +' z',
                          'transform':'rotate('+str(rotate)+') translate('\
                          +str(cx)+','+str(cy)+')'}
        inkex.etree.SubElement(parent, inkex.addNS('path','svg'), regmark_attribs)

class Printing_Marks (inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)
        self.OptionParser.add_option("--tab",
                        	     action="store", type="string", 
                        	     dest="tab", default="sampling",
                        	     help="The selected UI-tab when OK was pressed") 
        self.OptionParser.add_option("--crop_marks",
                                     action="store", type="inkbool",
                                     dest="crop_marks", default=True,
                                     help="Draw crop Marks?")
        self.OptionParser.add_option("--bleed_marks",
                                     action="store", type="inkbool",
                                     dest="bleed_marks", default=False,
                                     help="Draw Bleed Marks?")
        self.OptionParser.add_option("--registration_marks",
                                     action="store", type="inkbool",
                                     dest="reg_marks", default=False,
                                     help="Draw Registration Marks?")
        self.OptionParser.add_option("--colour_bars",
                                     action="store", type="inkbool",
                                     dest="colour_bars", default=False,
                                     help="Draw Colour Bars?")
        self.OptionParser.add_option("--page_info",
                                     action="store", type="inkbool",
                                     dest="page_info", default=False,
                                     help="Draw Page Information?")
        self.OptionParser.add_option("--unit",
                                     action="store", type="string",
                                     dest="unit", default=100.0,
                                     help="Draw measurment")
        self.OptionParser.add_option("--crop_offset",
                                     action="store", type="float",
                                     dest="crop_offset", default=0,
                                     help="Offset")
        self.OptionParser.add_option("--bleed_top",
                                     action="store", type="float",
                                     dest="bleed_top", default=0,
                                     help="Bleed Top Size")
        self.OptionParser.add_option("--bleed_bottom",
                                     action="store", type="float",
                                     dest="bleed_bottom", default=0,
                                     help="Bleed Bottom Size")
        self.OptionParser.add_option("--bleed_left",
                                     action="store", type="float",
                                     dest="bleed_left", default=0,
                                     help="Bleed Left Size")
        self.OptionParser.add_option("--bleed_right",
                                     action="store", type="float",
                                     dest="bleed_right", default=0,
                                     help="Bleed Right Size")


    def effect(self):

        # Get SVG document dimensions
        svg = self.document.getroot()
        width  = inkex.unittouu(svg.get('width'))
        height = inkex.unittouu(svg.attrib['height'])

        # Default parameters
        mark_th = 1
        mark = inkex.unittouu('7mm')
        rm_margin = inkex.unittouu('0mm')
        rm_center = (mark+rm_margin)/2
        hrm_length = (mark-rm_margin)/2
        rm_radius = hrm_length * 0.9
        # Colours are inner to outer
        rm_colours = ['#000000','#00ffff','#ff00ff','#ffff00','#000000']

        # Convert parameters to user unit
        offset = inkex.unittouu(str(self.options.crop_offset)\
                                +str(self.options.unit))
        bt = inkex.unittouu(str(self.options.bleed_top)\
                            +str(self.options.unit))
        bb = inkex.unittouu(str(self.options.bleed_bottom)\
                            +str(self.options.unit))
        bl = inkex.unittouu(str(self.options.bleed_left)\
                            +str(self.options.unit))
        br = inkex.unittouu(str(self.options.bleed_right)\
                            +str(self.options.unit))
        # Define the new document limits
        left = - mark - bl
        right = width + mark + br
        top = - mark - bt
        bottom = height + mark + bb

        # Create a new layer
        layer = inkex.etree.SubElement(svg, 'g')
        layer.set(inkex.addNS('label', 'inkscape'), 'Crop marks layer')
        layer.set(inkex.addNS('groupmode', 'inkscape'), 'layer')

        # Crop Mark
        if self.options.crop_marks == True:
            # Create a group for Crop Mark
            g_attribs = {inkex.addNS('label','inkscape'):'CropMarks'}
            g_crops = inkex.etree.SubElement(layer, 'g', g_attribs)

            # Top left Mark
            draw_SVG_line(0, top - offset, 0, top - offset + mark, mark_th,
                          'TLM1', g_crops)
            draw_SVG_line(left - offset, 0, left - offset + mark, 0, mark_th,
                          'TLM2', g_crops)

            # Top right Mark
            draw_SVG_line(width, top  - offset, width , top  - offset + mark,
                          mark_th, 'TRM1', g_crops)
            draw_SVG_line(right + offset, 0, right + offset - mark, 0, mark_th,
                          'TRM2', g_crops)

            # Bottom left Mark
            draw_SVG_line(0, bottom + offset, 0 , bottom + offset - mark,
                          mark_th, 'BLM1', g_crops)
            draw_SVG_line(left - offset, height, left - offset + mark, height,
                          mark_th, 'BLM2', g_crops)

            # Bottom right Mark
            draw_SVG_line(width, bottom + offset, width, bottom + offset - mark,
                          mark_th, 'BRM1', g_crops)
            draw_SVG_line(right + offset, height, right + offset - mark, height,
                          mark_th, 'BRM2', g_crops)

        # Bleed Mark
        if self.options.bleed_marks == True:
            # Create a group for Bleed Mark
            g_attribs = {inkex.addNS('label','inkscape'):'BleedMarks'}
            g_bleed = inkex.etree.SubElement(layer, 'g', g_attribs)

            # Top left Mark
            draw_SVG_dashedline(left + mark, top - offset,
                                left + mark, top - offset + mark,
                                mark_th, 'TLB1', g_bleed)
            draw_SVG_dashedline(left - offset, top + mark,
                                left - offset + mark, top + mark,
                                mark_th, 'TLB2', g_bleed)

            # Top right Mark
            draw_SVG_dashedline(right - mark, top - offset,
                                right - mark , top - offset + mark,
                                mark_th, 'TRB1', g_bleed)
            draw_SVG_dashedline(right + offset, top + mark,
                                right + offset - mark, top + mark,
                                mark_th, 'TRB2', g_bleed)

            # Bottom left Mark
            draw_SVG_dashedline(left + mark, bottom + offset,
                                left + mark , bottom + offset - mark,
                                mark_th, 'BLB1', g_bleed)
            draw_SVG_dashedline(left - offset, bottom - mark,
                                left - offset + mark, bottom - mark,
                                mark_th, 'BLB2', g_bleed)   

            # Bottom right Mark
            draw_SVG_dashedline(right - mark, bottom + offset,
                                right - mark, bottom + offset - mark,
                                mark_th, 'BRB1', g_bleed)
            draw_SVG_dashedline(right + offset, bottom - mark,
                                right + offset - mark, bottom - mark,
                                mark_th, 'BRB2', g_bleed)

        # Registration Mark
        if self.options.reg_marks == True:
            # Create a group for Registration Mark
            g_attribs = {inkex.addNS('label','inkscape'):'RegistrationMarks'}
            g_center = inkex.etree.SubElement(layer, 'g', g_attribs)

            # Left Mark
            g_attribs = {inkex.addNS('label','inkscape'):'Left Reg. Mark'}
            g_lrm = inkex.etree.SubElement(g_center, 'g', g_attribs)
            draw_SVG_RegMarks(left - offset + rm_center, (height / 2),
                              rm_radius, '0', 'LC', rm_colours, g_lrm)

            # Top Mark
            g_attribs = {inkex.addNS('label','inkscape'):'Top Reg. Mark'}
            g_trm = inkex.etree.SubElement(g_center, 'g', g_attribs)
            draw_SVG_RegMarks((top - offset + rm_center),-(width / 2),
                                rm_radius,'90', 'TC', rm_colours, g_trm)

            # Right Mark
            g_attribs = {inkex.addNS('label','inkscape'):'Right Reg. Mark'}
            g_rrm = inkex.etree.SubElement(g_center, 'g', g_attribs)
            draw_SVG_RegMarks(-(right + offset - rm_center), -(height / 2),
                                rm_radius,'180', 'RC', rm_colours, g_rrm)

            # Bottom Mark
            g_attribs = {inkex.addNS('label','inkscape'):'Bottom Reg. Mark'}
            g_brm = inkex.etree.SubElement(g_center, 'g', g_attribs)
            draw_SVG_RegMarks(-(bottom + offset - rm_center), (width / 2),
                                rm_radius,'-90', 'BC', rm_colours, g_brm)

        # Colour Bars
        if self.options.colour_bars == True:
            # To be added
            pass


        # Page Information
        if self.options.page_info == True:
            # To be added
            pass


if __name__ == '__main__':
    e = Printing_Marks()
    e.affect()
